<?php
/**
 * AlphaPagination class file.
 * 
 * AlphaPagination represents information relevant to AlphaPageLinker.
 * Strongly based on Qiang Xues {@link CPagination} from yiiframework.
 * 
 * @author Jascha Koch
 * @license http://www.yiiframework.com/license/
 * @version 1.0
 */

class AlphaPagination extends CComponent
{
	/**
	 * @var string name of the GET variable storing the current page index. Defaults to 'page'.
	 */
	public $pageVar='alpha';
	/**
	 * @var string name of the GET variable which indicates that a controlled pagination should 
	 * be reseted to the default page. Defaults to 'p_rst'.
	 */
	public $pageResetVar='p_rst';
	/**
	 * @var string the route (controller ID and action ID) for displaying the paged contents.
	 * Defaults to empty string, meaning using the current route.
	 */
	public $route='';
	/**
	 * @var array the additional GET parameters (name=>value) that should be used when generating pagination URLs.
	 * Defaults to null, meaning using the currently available GET parameters.
	 */
	public $params;
	
	/**
	 * @var string the model attribute that the condition should be applied to.
	 */
	public $attribute;
	
	/**
	 * @var string first character of pager range. Defaults to 'a'.
	 */
	public $startChar='a';
	
	/**
	 * @var string last character of pager range. Defaults to 'z'.
	 */
	public $endChar='z';

	private $_pagination;
	private $_currentPage;
	private $_chars;

	/**
	 * Constructor.
	 * @param string model attribute to which the search condition is applied.
	 * @param CPagination Pagination controlled per letter.
	 */
	public function __construct($attribute='',$pagination=null)
	{
		$this->attribute = $attribute;
		if($pagination!=null)
			$this->_pagination = $pagination;
	}

	/**
	 * @return integer the zero-based index of the current page. Defaults to 0.
	 */
	public function getCurrentPage()
	{
		if($this->_currentPage===null)
		{
			if(isset($_GET[$this->pageVar]))
			{
				$this->_currentPage=(int)$_GET[$this->pageVar];
				if($this->_currentPage<0 || $this->_currentPage>count($this->getCharRange()))
					$this->_currentPage=0;
			}
			else
				$this->_currentPage=0;
		}
		return $this->_currentPage;
	}

	/**
	 * @param integer the zero-based index of the current page.
	 */
	public function setCurrentPage($value)
	{
		$this->_currentPage=$value;
		$_GET[$this->pageVar]=$value;
	}

	/**
	 * Creates the URL suitable for pagination.
	 * This method is mainly called by pagers when creating URLs used to
	 * perform pagination. The default implementation is to call
	 * the controller's createUrl method with the page information.
	 * You may override this method if your URL scheme is not the same as
	 * the one supported by the controller's createUrl method.
	 * @param CController the controller that will create the actual URL
	 * @param integer the page that the URL should point to. This is a zero-based index.
	 * @return string the created URL
	 */
	public function createPageUrl($controller,$page)
	{
		$params=$this->params===null ? $_GET : $this->params;
		if($page>0) // page 0 is the default
			$params[$this->pageVar]=$page;
		else
			unset($params[$this->pageVar]);
			
		if($this->_pagination!=null)
			$params[$this->pageResetVar] = 1;
			
		return $controller->createUrl($this->route,$params);
	}

	/**
	 * Applies CONDITION to the specified query criteria.
	 * @param CDbCriteria the query criteria that should be applied with the limit
	 */
	public function applyCondition($criteria)
	{
		if($this->attribute!='')
		{
			$this->resetPaginationVar();
			$currentPage = $this->getCurrentPage();
			$search = $this->getChar($currentPage-1);
			
			if($currentPage>0 && $search!=null)
				$criteria->addSearchCondition($this->attribute,$search.'%',false);
		}
		else
			throw new CException(Yii::t('AlphaPagination','There is no value set for $attribute. You must set the model attribute the pagination condition should be applied to.'));
	}
	
	/**
	 * Creates the range of characters for pagination.
	 * @return array index based array with characters from {@see $startChar} to {@see $endChar}.
	 */
	public function getCharRange()
	{
		if($this->_chars===null)
			$this->_chars = range($this->startChar,$this->endChar);
		return $this->_chars;
	}
	
	/**
	 * @param integer ordinal number of the character. zero-based indexing.
	 * @return string character of the char range. NULL if out of range.
	 */
	public function getChar($ord)
	{
		$chars = $this->getCharRange();
		return ($ord>=0&&$ord<count($chars))?$chars[$ord]:null;
	}
	
	/**
	 * Unsets the $pageVar of a controlled CPagination to make it fall back to default page.
	 */
	private function resetPaginationVar()
	{
		if($this->_pagination!=null && isset($_GET[$this->pageResetVar]))
			unset($_GET[$this->pageResetVar],$_GET[$this->pagination->pageVar]);
	}
	
	/**
	 * @param value CPagination the controlled CPagination object 
	 */
	public function setPagination($value)
	{
		$this->_pagination = $value;
	}
	
	/**
	 * @return CPagination the controlled CPagination object 
	 */
	public function getPagination()
	{
		if($this->_pagination===null)
			$this->_pagination=new CPagination();
		return $this->_pagination;
		
	}
}
?>